"""network_importer driver for arista_eos."""

from .default import NautobotNornirDriver as DefaultNautobotNornirDriver


class NautobotNornirDriver(DefaultNautobotNornirDriver):
    """Collection of Nornir Tasks specific to Arista EOS devices."""



"""Nornir Nautobot Inventory Plugin."""
# Python Imports
import os
import sys
import ipaddress
import logging
from typing import Any, Dict, Union

# Nornir Imports
from nornir.core.inventory import (
    Inventory,
    ConnectionOptions,
    Defaults,
    Groups,
    Host,
    Hosts,
)

# Other third party imports
import pynautobot
from requests import Session

# Create Logger
logger = logging.getLogger(__name__)


def _set_host(data: Dict[str, Any], name: str, groups, host, host_platform) -> Host:
    connection_option = {}
    for key, value in data.get("connection_options", {}).items():
        connection_option[key] = ConnectionOptions(
            hostname=value.get("hostname"),
            port=value.get("port"),
            username=value.get("username"),
            password=value.get("password"),
            platform=host_platform,
            extras=value.get("extras"),
        )

    return Host(
        name=name,
        hostname=host["hostname"],
        username=host.get("username"),
        password=host.get("password"),
        platform=host_platform,
        data=data,
        groups=groups,
        connection_options=connection_option,
    )


# Setup connection to Nautobot
class NautobotInventory:
    """Nautobot Nornir Inventory."""

    def __init__(
        self,
        nautobot_url: Union[str, None],
        nautobot_token: Union[str, None],
        ssl_verify: Union[bool, None] = True,
        filter_parameters: Union[Dict[str, Any], None] = None,
    ) -> None:
        """Nautobot nornir class initialization."""
        self.nautobot_url = nautobot_url or os.getenv("NAUTOBOT_URL")
        self.nautobot_token = nautobot_token or os.getenv("NAUTOBOT_TOKEN")
        self.filter_parameters = filter_parameters
        self.ssl_verify = ssl_verify
        self._verify_required()
        self._api_session = None
        self._devices = None
        self._pynautobot_obj = None

    def _verify_required(self) -> bool:
        """Verify that required parameters are provided either passed in or via environment.

        Returns:
            bool: Successful

        Raises:
            ValueError: When incorrect value is provided
        """
        for item in [self.nautobot_url, self.nautobot_token]:
            if item is None:
                raise ValueError("Missing URL or Token from parameters or environment.")

        return True

    @property
    def api_session(self):
        """Requests session to pass into Nautobot."""
        if self._api_session is None:
            self._api_session = Session()
            self._api_session.verify = self.ssl_verify

        return self._api_session

    @property
    def pynautobot_obj(self) -> pynautobot.core.api.Api:
        """Pynautobot API object to interact with Nautobot.

        Returns:
            pynautobot object: Object to interact with the pynautobot SDK.
        """
        if self._pynautobot_obj is None:
            self._pynautobot_obj = pynautobot.api(self.nautobot_url, token=self.nautobot_token)
            self._pynautobot_obj.http_session = self.api_session

        return self._pynautobot_obj

    @property
    def devices(self) -> list:
        """Devices information from Nautobot."""
        if self._devices is None:
            # Check for filters. Cannot pass an empty dictionary to the filter method
            if self.filter_parameters is None:
                self._devices = self.pynautobot_obj.dcim.devices.all()
            else:
                try:
                    self._devices = self.pynautobot_obj.dcim.devices.filter(**self.filter_parameters)
                except pynautobot.core.query.RequestError:
                    print("Error in the query filters. Please verify the parameters.")
                    sys.exit(1)

        return self._devices

    # Build the inventory
    def load(self) -> Inventory:
        """Load of Nornir inventory.

        Returns:
            Inventory: Nornir Inventory
        """
        hosts = Hosts()
        groups = Groups()
        defaults = Defaults()

        for device in self.devices:
            # Set the base information for a device
            host: Dict[Any, Any] = {"data": {}}

            # Assign the pynautobot host object to the data key
            host["data"]["pynautobot_object"] = device

            # Create dictionary object available for filtering
            host["data"]["pynautobot_dictionary"] = dict(device)
            # TODO: #3 Investigate Nornir compatability with dictionary like object

            # Add Primary IP address, if found. Otherwise add hostname as the device name
            host["hostname"] = (
                str(ipaddress.IPv4Interface(device.primary_ip.address).ip) if device["primary_ip"] else device["name"]
            )
            host["name"] = device.name or str(device.id)
            host["groups"] = []

            # Add host to hosts by name first, ID otherwise - to string
            hosts[device.name or str(device.id)] = _set_host(  # pylint: disable=unsupported-assignment-operation
                data=host["data"],
                name=host["name"],
                groups=host["groups"],
                host=host,
                host_platform=getattr(device.platform, "slug", None),
            )

        return Inventory(hosts=hosts, groups=groups, defaults=defaults)



"""network_importer driver for cisco NXOS."""

from .default import NetmikoNautobotNornirDriver as DefaultNautobotNornirDriver


class NautobotNornirDriver(DefaultNautobotNornirDriver):
    """Driver for Cisco NXOS."""



"""Provide logging facility for Nautobot/Python usage."""

import logging


class NornirLogger:
    """Similar to a mixin, to utilize Python logging and Jobs Result obj."""

    def __init__(self, name, job_result=None, debug=False):
        """Initialize the object."""
        self.job_result = job_result
        self.logger = logging.getLogger(name)
        self.debug = debug

    def log_debug(self, message):
        """Debug, does not take obj, and only logs to jobs result when in global debug mode."""
        if self.job_result and self.debug:
            self.job_result.log_debug(message)
        self.logger.debug(message)

    def log_info(self, obj, message):
        """Log to Python logger and jogs results for info messages."""
        if self.job_result:
            self.job_result.log_info(obj, message)
        self.logger.info("%s | %s", str(obj), message)

    def log_success(self, obj, message):
        """Log to Python logger and jogs results for success messages."""
        if self.job_result:
            self.job_result.log_success(obj, message)
        self.logger.info("%s | %s", str(obj), message)

    def log_warning(self, obj, message):
        """Log to Python logger and jogs results for warning messages."""
        if self.job_result:
            self.job_result.log_warning(obj, message)
        self.logger.warning("%s | %s", str(obj), message)

    def log_failure(self, obj, message):
        """Log to Python logger and jogs results for failure messages."""
        if self.job_result:
            self.job_result.log_failure(obj, message)
        self.logger.error("%s | %s", str(obj), message)



"""default network_importer driver for Juniper."""

from .default import NetmikoNautobotNornirDriver as DefaultNautobotNornirDriver


class NautobotNornirDriver(DefaultNautobotNornirDriver):
    """Collection of Nornir Tasks specific to Juniper Junos devices."""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nornir_nautobot',
 'nornir_nautobot.plugins',
 'nornir_nautobot.plugins.inventory',
 'nornir_nautobot.plugins.processors',
 'nornir_nautobot.plugins.tasks.dispatcher',
 'nornir_nautobot.utils']

package_data = \
{'': ['*']}

install_requires = \
['netutils>=0.1.2,<0.2.0',
 'nornir-jinja2>=0.1.1,<0.2.0',
 'nornir-napalm>=0.1.1,<0.2.0',
 'nornir-netmiko>=0.1.1,<0.2.0',
 'nornir-utils>=0.1.1,<0.2.0',
 'nornir>=3.0.0,<4.0.0',
 'pynautobot>=1.0.1,<2.0.0',
 'requests>=2.25.1,<3.0.0']

entry_points = \
{'nornir.plugins.inventory': ['NautobotInventory = '
                              'nornir_nautobot.plugins.inventory.nautobot:NautobotInventory']}

setup_kwargs = {
    'name': 'nornir-nautobot',
    'version': '2.0.1',
    'description': 'Nornir Nautobot',
    'long_description': "# nornir_nautobot\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n\n## Build Status\n\n| Branch  | Status                                                                                                                               |\n| ------- | ------------------------------------------------------------------------------------------------------------------------------------ |\n| main    | [![Build Status](https://travis-ci.com/nautobot/nornir-nautobot.svg?branch=main)](https://travis-ci.com/nautobot/nornir-nautobot)    |\n| develop | [![Build Status](https://travis-ci.com/nautobot/nornir-nautobot.svg?branch=develop)](https://travis-ci.com/nautobot/nornir-nautobot) |\n## Overview\n\nThe nornir_nautobot project intends to solve two primary use cases.\n\n* Providing a Nornir inventory that leverages Nautobot's API.\n* A set of opinionated Nornir plugins.\n\nThe set of plugins intend to provide mechanisms to include common networking workflows that will help enable network automation. As an example, there are method to get configurations or test network connectivity. Over time this will include functions to perform actions such as get vlans, neighbors, protocols, etc.\n\n## Getting Started\n\n```shell\npip install nornir-nautobot\n```\n\n## Documentation Link\n\nThe documentation can be found on [Read the Docs](https://nornir-nautobot.readthedocs.io/en/latest/)\n\n## Nautobot\n\nNautobot documentation is available at [Nautobot Read the Docs](https://nautobot.readthedocs.io/en/latest/)\n",
    'author': 'Network to Code, LLC',
    'author_email': 'opensource@networktocode.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://nautobot.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
